;------------------------------------;;-------------------------->
; Read the Temporary BBY instrument files (449 MHz first, others later)
; Perform processing tasks (derived variables), assign to arrays
; and write a NetCDF file.
;
; This will become version 2. It is being updated to accommodate
; missing time periods in the PSD archive. Currently, it can
; ingest surface met data, detect missing periods and appropriately
; distribute the data to a regular hourly timeseries.
;
; This needs to be generalized to the 449 MHz data, RASS and GPS.
; Additionally, if any of these measurements is completely missing
; from the day in question, it needs to be adjusted to simply write 
; a full 24-hours of missing values. 
;
; Finally, We will add an option to ingest radiosonde data and 
; interpolate it to 449/RASS levels and closest hourly observation.
;
;----------------------------;;---------------------------------->
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
;----------------------------;;---------------------------------->
function CW3Esonde_2_AROvars(filename:string, zlevs:numeric ,nlevs[1] ,varname:string)
; 
; interpolate sonde variables to ARO radar levels 
; and calculate diagnostic variables (e.g. Theta-v)
;  
; filename: name of file to read [ type: string, shape: scalar ]
; zlevs: levels to interpolate to [ type: float, shape: (/nlevs/) ]
; nlevs: number levels in output [ type: float, shape: scalar ]
; varname: name of output variable (controls physical manipulation) [ type: string, shape: scalar ]
; 
local inpt, datain, nlines, nlvl, pin, tin, zin, tin, rhin, dirin, spdin, uin, vin, \
       qin, latin, lonin, tempvar, tolerance 
begin
  ;print("converting to "+varname+" and interpolating to "+nlevs+" levels.")
  outvar = new((/nlevs/),float)
  
  tolerance = avg(center_finite_diff(zlevs,1.0,False,0))/4.
  ;print("average out level spacing = "+tolerance)
  
  
;------------------------------------------------>
; Now read sonde  
  inpt = asciiread(filename,-1,"string")
  nlines = dimsizes(inpt)

  datain = inpt(14:nlines-3)
  
  nlvl = dimsizes(datain)
  
  pin = new((/nlvl/),float)
  zin = pin
  tin = pin
  rhin = pin
  dirin = pin
  spdin = pin
  uin   = pin
  vin   = pin
  latin = pin
  lonin = pin
  
  do iline = 0,nlvl-1
    temp = str_split(datain(iline), " ")
    pin(iline) = tofloat(temp(1))
    zin(iline) = tofloat(temp(2))
    tin(iline) = tofloat(temp(3))
    rhin(iline) = tofloat(temp(4))
    dirin(iline) = tofloat(temp(5))
    spdin(iline) = tofloat(temp(6))
    uin(iline) = tofloat(temp(7))
    vin(iline) = tofloat(temp(8))
    latin(iline) = tofloat(temp(9))
    lonin(iline) = tofloat(temp(10))
  end do
  
  qin = mixhum_ptrh(pin, tin + 273.15, rhin, 1)
  
;------------------------------------------------>
; Now make (optinal) variable conversion and interpolate
  if(varname.eq."u")then
   tempvar = uin
  end if 
  
  if(varname.eq."v")then
   tempvar = vin
  end if
  
  if(varname.eq."qv")then
   tempvar = qin
  end if
  
  if(varname.eq."t")then
   tempvar = tin
  end if
  
  if(varname.eq."p")then
   tempvar = pin
  end if
  
  if(varname.eq."tv")then
   tempvar = tin*(1.0 + 0.61*qin)
  end if  
  
  if(varname.eq."thv")then
   tempvar = pot_temp(pin*100.,(tin*(1.0 + 0.61*qin) + 273.),-1,False)
  end if 

  
;------------------------------------------------>
; Now  interpolate
   do ilev = 0,nlevs-1
    dxes = ind(zin.gt.(zlevs(ilev)-tolerance)*1000..and.zin.lt.(zlevs(ilev)+tolerance)*1000.)
;    print("for ilev = "+ilev+", lower height = "+(zlevs(ilev)-tolerance)*1000.+", upper height = "+(zlevs(ilev)+tolerance)*1000.)
    if(any(ismissing(dxes)))then
     outvar(ilev) = outvar@_FillValue
    else
     outvar(ilev) = avg(tempvar(dxes))
    end if 
     delete(dxes)
   end do
  

return(outvar)
end


begin

err = NhlGetErrorObjectId()
  setvalues err
    "errLevel" : "Fatal"          ; only report Fatal errors
  end setvalues

;------------------------------;;--------------------------------->
; declare station metadata
;------------------------------;;--------------------------------->

;longitude = LONN
longitude = -123.0728
longitude@units = "degrees_east"
longitude@description = "SSS station longitude"
;latitude  = LATT
latitude  = 38.3191
latitude@units = "degrees_north"
latitude@description = "SSS station latitude"
;altitude  = ALTT
elevation  = 15.
elevation@units = "m MSL"
elevation@decsription = "SSS station elevation" 
;upslope   = DEGG
upslope   = 230.
upslope@units = "degrees"
upslope@description = "orographic upslope direction from SSS station"

;=========================================>>
; Declare time-base metadata

nhrs = 24
year = "YRR"
month = "MNN"
day = "DDY"

;-------------------------------;;-------------------------------->
; declare arrays
;-------------------------------;;-------------------------------->

;------------------------------->>
; Get retr01 template levels
retr01_template_file = "/home/mc/Special-NCL-Funcs/Null-BBY-retr01.txt"

x1 = asciiread(retr01_template_file,-1,"string")
nlines = dimsizes(x1)
nzp = nlines/2
retr01_levs = new((/nzp/),float)


do iline = 0,nzp-1
    temp = str_split(x1(iline), " ")
    retr01_levs(iline) = tofloat(temp(1))
end do    
delete(x1)
delete(temp)

;------------------------------->>
; Get retr02 template levels
retr02_template_file = "/home/mc/Special-NCL-Funcs/Null-BBY-retr02.txt"

x1 = asciiread(retr02_template_file,-1,"string")
retr02_levs = new((/nzp/),float)


do iline = 0,nzp-1
    temp = str_split(x1(iline), " ")
    retr02_levs(iline) = tofloat(temp(1))
end do    
delete(x1)
delete(temp)


;------------------------------->>
; Get rass template levels
rass_template_file = "/home/mc/Special-NCL-Funcs/Null-BBY-rass.txt"

x1 = asciiread(rass_template_file,-1,"string")
nlines = dimsizes(x1)
nzr = nlines/2
rass_levs = new((/nzr/),float)


do iline = 0,nzr-1
    temp = str_split(x1(iline), " ")
    rass_levs(iline) = tofloat(temp(1))
end do    
delete(x1)
delete(temp)





;;============================================>>
; Define baseline time vector for HMT data

units = "minutes after 2006-01-01 00:00:00"

yy = tointeger(year)
mm = tointeger(month)
dd = tointeger(day)
hh = 0
mn = 0
ss = 0

start = cd_inv_calendar(yy,mm,dd,hh,mn,ss,units,0)

fulltimes = new((/nhrs/),float)
fulltimes@_FillValue = -9999.
fulltimes@units = units
fulltimes = fspan(dble2flt(start),dble2flt(start)+60.*(nhrs-1),nhrs)




;; Surface Met

inpt_fill = -9999.

pres_sfc = new((/nhrs/),float)
pres_sfc@_FillValue = inpt_fill
pres_sfc@units = "hPa"
pres_sfc@desciption = "Hourly mean pressure at 2m from the near-surface barometer"

pres_sfc_2min = new((/nhrs/),float)
pres_sfc_2min@_FillValue = inpt_fill
pres_sfc_2min@units = "hPa"
pres_sfc_2min@desciption = "Instantaneous pressure at 2m from the near-surface barometer"

temp_sfc = new((/nhrs/),float)
temp_sfc@_FillValue = inpt_fill
temp_sfc@units = "degrees C"
temp_sfc@description = "Hourly mean temperature at 2m from the near-surface thermometer"

rh_sfc = new((/nhrs/),float)
rh_sfc@_FillValue = inpt_fill
rh_sfc@units = "%"
rh_sfc@description = "Hourly mean relative humidity from the near-surface hygrometer"

wspd_sfc = new((/nhrs/),float)
wspd_sfc@_FillValue = inpt_fill
wspd_sfc@units = "m s^-1"
wspd_sfc@description = "Hourly mean wind speed from 10m anemometer"

wdir_sfc = new((/nhrs/),float)
wdir_sfc@_FillValue = inpt_fill
wdir_sfc@units = "degrees"
wdir_sfc@units = "Hourly mean wind direction from 10m anemometer"

solrad_sfc = new((/nhrs/),float)
solrad_sfc@_FillValue = inpt_fill
solrad_sfc@units = "W m^-2"
solrad_sfc@description = "Hourly mean downwelling solar radiation at surface"

precip = new((/nhrs/),float)
precip@_FillValue = inpt_fill
precip@units = "mm"
precip@description = "Hourly accumulated precipitation from tipping bucket raingauge"

;; GPS Trimble

gpsiwv = new((/nhrs/),float)
gpsiwv@_FillValue = inpt_fill
gpsiwv@units = "mm"
gpsiwv@description = "Hourly average Integrated Water Vapor from GPS Trimble"


;; Wind Profiler
z449 = new((/nhrs,nzp,2/),float)
z449@_FillValue = inpt_fill
mag449 = z449
dir449 = z449
qc449 = z449



;; RASS (zrass, trass, wrass, qc_trass, qc_wrass (/nhrs,nzr/))
zrass = new((/nhrs,nzr/),float)
zrass@_FillValue = -9999.
zrass@units = "km AGL"
zrass@description = "Height of RASS vertical level"

trass = new((/nhrs,nzr/),float)
trass@_FillValue = -9999.
trass@units = "degrees C"
trass@units = "Corrected virtual temperature estimate from RASS"

wrass = new((/nhrs,nzr/),float)
wrass@_FillValue = -9999.
wrass@units = "m s^-1"
wrass@description = "Vertical velocity estimate from RASS" 

qc_trass = new((/nhrs,nzr/),float)
qc_trass@_FillValue = -9999.
qc_trass@units = "None"
qc_trass@description = "qc flag for RASS temp retrieval. 0 = good, 9 = rejected"

qc_wrass = new((/nhrs,nzr/),float)
qc_wrass@_FillValue = -9999.
qc_wrass@units = "None"
qc_wrass@description = "qc flag for RASS temp retrieval. 0 = good, 9 = rejected"



;; Derived Fields
thrass = new((/nhrs,nzr/),float)
thrass@_FillValue = -9999.
thrass@units = "K"
thrass@description = "Derived virtual potential temperature from RASS retrieval"

prass = new((/nhrs,nzr/),float)
prass@_FillValue = -9999.
prass@units = "hPa"
prass@units = "Derived hydrostatic pressure at RASS vertical level"

bulkiwvflux = new((/nhrs/),float)
bulkiwvflux@_FillValue = -9999.
bulkiwvflux@units = "mm m s^-1"
bulkiwvflux@description = "Water vapor flux in the upslope direction (Neiman 2009)"

;; Internal diagnostics
pres_sfc_2min = new((/nhrs/),float)
pres_sfc_2min@_FillValue = -9999.

dzrass = new((/nhrs,nzr/),float)
dzrass@_FillValue = -9999.


;------------------------------;;--------------------------------->
; Pre-Process Surface Met
;------------------------------;;--------------------------------->

; Note, wildcards will be used below: bby -> SSS, 2015 -> YRR, 014 -> DYR
filename = "./Temp-SSS-YRRDYR-sfcmet.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

juldd = new((/nlines/),integer)
hhin = new((/nlines/),string)
hhout = new((/nlines/),float)
pin = new((/nlines/),float)
pin@_FillValue = 999999
tin = pin
rhin = pin
dirin = pin
spdin = pin
sradin = pin
prcpin = pin

do iline = 0,nlines-1
    temp = str_split(x1(iline), " ")
    
    juldd(iline) = tointeger(temp(0))
    tempstr = temp(1)
    
    if(strlen(tempstr).eq.1)then
    hhin(iline) = str_get_cols("000"+tempstr,0,1)
    end if
    
    if(strlen(tempstr).eq.2)then
    hhin(iline) = str_get_cols("00"+tempstr,0,1)
    end if
    
    if(strlen(tempstr).eq.3)then
    hhin(iline) = str_get_cols("0"+tempstr,0,1)
    end if
    
    if(strlen(tempstr).eq.4)then
    hhin(iline) = str_get_cols(tempstr,0,1)
    end if
    
    
    pin(iline) = tofloat(temp(2))
    tin(iline) = tofloat(temp(3))
    rhin(iline) = tofloat(temp(4))
    spdin(iline) = tofloat(temp(5))
    dirin(iline) = tofloat(temp(6))
    sradin(iline) = tofloat(temp(7))
    prcpin(iline) = tofloat(temp(8))
    
    hhout(iline) = dble2flt(cd_inv_calendar(yy,mm,dd,tointeger(hhin(iline)),0,0,units,0))
end do



if(all(ismissing(pin)))then
 continue
else 

;; Heres where the mods are required. Can no longer assume input data is on a 
;; regular time interval without missing periods.


do ihr = 0,nhrs-1
 dxes = ind(hhout.eq.fulltimes(ihr))
if(.not.any(ismissing(dxes)))then 
 pres_sfc(ihr) = avg(pin(dxes))
 temp_sfc(ihr) = avg(tin(dxes))
 rh_sfc(ihr) = avg(rhin(dxes))
 wspd_sfc(ihr) = avg(spdin(dxes))
 wdir_sfc(ihr) = avg(dirin(dxes))
 solrad_sfc(ihr) = avg(sradin(dxes))
 precip(ihr) = sum(prcpin(dxes))
 pres_sfc_2min(ihr) = pin(min(dxes))
end if
 delete(dxes)
end do

delete(temp)
end if

delete(x1)
delete(juldd) 
delete(hhin) 
delete(hhout)
delete(pin)
delete(tin) 
delete(rhin) 
delete(dirin)
delete(spdin) 
delete(sradin)
delete(prcpin)



;; Sanity Checks

;print("max pres = "+max(pres_sfc_2min)+" min pres = "+min(pres_sfc_2min))
;print("max temp = "+max(temp_sfc)+" min temp = "+min(temp_sfc))
;print("max rh = "+max(rh_sfc)+" min rh = "+min(rh_sfc))
;print("max wspd = "+max(wspd_sfc)+" min wspd = "+min(wspd_sfc))
;print("max wdir = "+max(wdir_sfc)+" min wdir = "+min(wdir_sfc))
;print("max solrad = "+max(solrad_sfc)+" min solrad = "+min(solrad_sfc))
;print("max precip = "+max(precip)+" min precip = "+min(precip))


;------------------------------;;--------------------------------->
; Now pre-process GPS IWV 
;------------------------------;;--------------------------------->

; Note, wildcards will be used below: bby -> SSS, 2015 -> YRR, 014 -> DYR
filename = "./Temp-SSS-YRRDYR-gpsiwv.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

hhin = new((/nlines/),integer)
hhout = new((/nlines/),float)
iwvin = new((/nlines/),float)
iwvin@_FillValue = -9.99



do iline = 0,nlines-1
    temp = str_split(x1(iline), " ")
    hhin(iline) = tointeger(temp(0))
    iwvin(iline) = tofloat(temp(1))
    
    hhout(iline) = dble2flt(cd_inv_calendar(yy,mm,dd,hhin(iline),0,0,units,0))
end do 
delete(temp)

if(all(ismissing(iwvin)))then
 print("no valid gps iwv data")
 continue
else
    
do ihr = 0,nhrs-1
 dxes = ind(hhout.eq.fulltimes(ihr))
if(.not.any(ismissing(dxes)))then 
 qpsiwv(ihr) = avg(iwvin(dxes))
end if
 delete(dxes)
end do



gpsiwv = mask(gpsiwv, gpsiwv.eq.-9.99, False)*10.0
end if

delete(hhin) 
delete(hhout)
delete(x1)
delete(iwvin)



opt = True
opt@PrintStat = True

;; Sanity Check
;stats = stat_dispersion(gpsiwv,opt)


;------------------------------;;--------------------------------->
; Similarly for 449 MHz 
;------------------------------;;--------------------------------->

filename = "./Temp-SSS-YRRDYR-retr01.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

hhin = new((/nlines/),integer)
hhout = new((/nlines/),float)
levin = new((/nlines/),float)
spdin = new((/nlines/),float)
spdin@_FillValue = 999999
dirin = spdin
qcin = spdin


do iline = 0,nlines-1
    temp = str_split(x1(iline), " ")
    hhin(iline) = tointeger(temp(0))
    levin(iline) = tofloat(temp(1))
    spdin(iline) = tofloat(temp(2))
    dirin(iline) = tofloat(temp(3))
    qcin(iline) = tofloat(temp(4))

    hhout(iline) = dble2flt(cd_inv_calendar(yy,mm,dd,hhin(iline),0,0,units,0))
end do

delete(temp)

if(all(ismissing(spdin)))then
 print("no valid 449 retr01 data")
 continue
else
    
 do ihr = 0,nhrs-1
   dxes = ind(hhout.eq.fulltimes(ihr))
  if(.not.any(ismissing(dxes)))then 
   mag449(ihr,:,0) = spdin(dxes)
   dir449(ihr,:,0) = dirin(dxes)
   qc449(ihr,:,0) = qcin(dxes)
  end if
   z449(ihr,:,0) = retr01_levs(:)
   delete(dxes)
    
 end do
end if



delete(hhin) 
delete(hhout)
delete(levin)
delete(spdin)
delete(dirin)
delete(qcin)
delete(x1)


;; Sanity Check
;print(""+mag449(:,2,0))


;------------------------------;;--------------------------------->
; Similarly for 449 MHz retr02
;------------------------------;;--------------------------------->

filename = "./Temp-SSS-YRRDYR-retr02.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

hhin = new((/nlines/),integer)
hhout = new((/nlines/),float)
levin = new((/nlines/),float)
spdin = new((/nlines/),float)
spdin@_FillValue = 999999
dirin = spdin
qcin = spdin


do iline = 0,nlines-1
    temp = str_split(x1(iline), " ")
    hhin(iline) = tointeger(temp(0))
    levin(iline) = tofloat(temp(1))
    spdin(iline) = tofloat(temp(2))
    dirin(iline) = tofloat(temp(3))
    qcin(iline) = tofloat(temp(4))

    hhout(iline) = dble2flt(cd_inv_calendar(yy,mm,dd,hhin(iline),0,0,units,0))
end do

delete(temp)

if(all(ismissing(spdin)))then
 print("no valid 449 retr02 data")
 continue
else
    
 do ihr = 0,nhrs-1
   dxes = ind(hhout.eq.fulltimes(ihr))
  if(.not.any(ismissing(dxes)))then 
   mag449(ihr,:,1) = spdin(dxes)
   dir449(ihr,:,1) = dirin(dxes)
   qc449(ihr,:,1) = qcin(dxes)
  end if
   z449(ihr,:,1) = retr02_levs(:)
   delete(dxes)
    
 end do
end if



delete(hhin) 
delete(hhout)
delete(levin)
delete(spdin)
delete(dirin)
delete(qcin)
delete(x1)





mag449@_FillValue = -9999.
dir449@_FillValue = -9999.

;; Sanity Check
;print(""+mag449(:,2,1))

;------------------------------;;--------------------------------->
; Similarly for RASS (zrass, trass, wrass, qc_trass, qc_wrass (/nhrs,nzr/))
;------------------------------;;--------------------------------->

filename = "./Temp-SSS-YRRDYR-rass.txt"

x1 = asciiread(filename,-1,"string")
nlines = dimsizes(x1)

hhin = new((/nlines/),integer)
hhout = new((/nlines/),float)
levin = new((/nlines/),float)
tin = new((/nlines/),float)
tin@_FillValue = 999999
win = tin
qctin = tin
qcwin = tin


do iline = 0,nlines-1
    temp = str_split(x1(iline), " ")
    hhin(iline) = tointeger(temp(0))
    levin(iline) = tofloat(temp(1))
    tin(iline) = tofloat(temp(2))
    win(iline) = tofloat(temp(3))
    qctin(iline) = tofloat(temp(4))
    qcwin(iline) = tofloat(temp(5))

    hhout(iline) = dble2flt(cd_inv_calendar(yy,mm,dd,hhin(iline),0,0,units,0))
end do

delete(temp)

if(all(ismissing(tin)))then
 print("no valid rass data")
 continue
else
    
 do ihr = 0,nhrs-1
   dxes = ind(hhout.eq.fulltimes(ihr))
  if(.not.any(ismissing(dxes)))then 
   trass(ihr,:) = tin(dxes)
   wrass(ihr,:) = win(dxes)
   qc_trass(ihr,:) = qctin(dxes)
   qc_wrass(ihr,:) = qcwin(dxes)
  end if
   zrass(ihr,:) = rass_levs(:)
   delete(dxes)
    
 end do
end if



delete(hhin) 
delete(hhout)
delete(levin)
delete(tin)
delete(win)
delete(qctin)
delete(qcwin)
delete(x1)

dzrass = center_finite_diff_n(zrass,1.,False,0,1) * 1000. ; convert to meters

trass@_FillValue = -9999.
wrass@_FillValue = -9999.

;; Sanity Check
;print(""+zrass(1,:))


;-------------------------------;;-------------------------------->
; Now, perform processing. For 449 MHz 
; This only involves a transform from spd,dir to u,v and replacing
; 999999 with a more robust missing value.
;
; For RASS, this involves both replacing missing values and 
; calculating the theta-v profile.
;
;-------------------------------;;-------------------------------->

z449_retr1 = new((/nhrs,nzp/),float)
z449_retr1@_FillValue = -9999.
z449_retr1@units = "km AGL"
z449_retr1@description = "Height of wind profiling radar vertical level for retrieval one"

z449_retr2 = new((/nhrs,nzp/),float)
z449_retr2@_FillValue = -9999.
z449_retr2@units = "km AGL"
z449_retr2@description = "Height of wind profiling radar vertical level for retrieval two"

u449_retr1 = new((/nhrs,nzp/),float)
u449_retr1@_FillValue = -9999.
u449_retr1@units = "m s^-1"
u449_retr1@description = "Zonal wind from profiling radar retrieval one"

u449_retr2 = new((/nhrs,nzp/),float)
u449_retr2@_FillValue = -9999.
u449_retr2@units = "m s^-1"
u449_retr2@description = "Zonal wind from profiling radar retrieval two"

v449_retr1 = new((/nhrs,nzp/),float)
v449_retr1@_FillValue = -9999.
v449_retr1@units = "m s^-1"
v449_retr1@description = "Meridional wind from profiling radar retrieval one"

v449_retr2 = new((/nhrs,nzp/),float)
v449_retr2@_FillValue = -9999.
v449_retr2@units = "m s^-1"
v449_retr2@description = "Meridional wind from profiling radar retrieval two"

qc449_retr1 = new((/nhrs,nzp/),float)
qc449_retr1@_FillValue = -9999.
qc449_retr1@units = "None"
qc449_retr1@description = "qc flag for 449 MHz retrieval. 0 = good, 9 = rejected"

qc449_retr2 = new((/nhrs,nzp/),float)
qc449_retr2@_FillValue = -9999.
qc449_retr2@units = "None"
qc449_retr2@description = "qc flag for 449 MHz retrieval. 0 = good, 9 = rejected"

z449_retr1 = (/z449(:,:,0)/)
z449_retr2 = (/z449(:,:,1)/)
qc449_retr1 = (/qc449(:,:,0)/)
qc449_retr2 = (/qc449(:,:,1)/)

uvtemp = wind_component(mag449,dir449,0)

u449_retr1 = (/uvtemp(0,:,:,0)/)
u449_retr2 = (/uvtemp(0,:,:,1)/)

v449_retr1 = (/uvtemp(1,:,:,0)/)
v449_retr2 = (/uvtemp(1,:,:,1)/)

delete(uvtemp)
delete(z449)
delete(qc449)


;----------------------------------;;------------------------------------->
; Now process derived variables for RASS**
;----------------------------------;;------------------------------------->



; make pressure profile first

trass = trass + 273.15  ;; convert virtual temp. to K

prass(:,0) = (/ pres_sfc_2min(:) /) ;; starts in hPa

do ihr = 0,nhrs-1
do ilev = 1,nzr-1
 prass(ihr,ilev) = prass(ihr,ilev-1) / (/ exp((9.81/(287.058*trass(ihr,ilev-1)))*dzrass(ihr,ilev-1)) /)
end do
end do

thrass = trass*((1000./prass)^0.286) 


trass = trass - 273.15  ;; back to degrees C


opt = True
opt@PrintStat = True
;stats = stat_dispersion(thrass,opt)

;print(trass(0,0))
;print("RASS height at time 1 = "+zrass(1,:))
;print("RASS press at time 1 = "+prass(1,:))

;-------------------------------;;---------------------------------->
; Calculate bulk upslope vapor flux
;-------------------------------;;---------------------------------->

component = u449_retr1
component = mag449(:,:,0)*cos((3.141593/180.)*(360. + dir449(:,:,0) - upslope))

component!0 = "times"
component!1 = "lev"
component&times = fulltimes
component&lev   = z449_retr1(0,:)

bulkiwvflux = gpsiwv * dim_avg_n(component(:,{0.75:1.25}),1) ; 

delete(component)
delete(mag449)
delete(dir449)



;---------------------------------;;---------------------------------->
; Handle Sondes
;---------------------------------;;---------------------------------->

;---------------;;--------------------------------------------------->
; Determine list of sonde files valid for requested date (YRRMNNDDY)

snd_list = systemfunc("ls ./sondes/raob"+sprinti("%0.4i",YRR)+sprinti("%0.2i",MNN)+sprinti("%0.2i",DDY)+"*.txt")
nsondes = dimsizes(snd_list)


sonde_times = new((/nsondes/),float)
sonde_times@_FillValue = -9999.
sonde_times@units = units
z_sonde_uv = new((/nzp/),float)
z_sonde_tv = new((/nzr/),float)

u_sonde = new((/nsondes,nzp/),float)
v_sonde = new((/nsondes,nzp/),float)
q_sonde = new((/nsondes,nzp/),float)
t_sonde = new((/nsondes,nzp/),float)
p_sonde = new((/nsondes,nzp/),float)

tv_sonde = new((/nsondes,nzr/),float)
thv_sonde = new((/nsondes,nzr/),float)
prass_sonde = new((/nsondes,nzr/),float)

z_sonde_uv = retr01_levs
z_sonde_tv = rass_levs



do isonde = 0,nsondes-1
yy = toint(str_get_cols(snd_list(isonde),13,16))
mm = toint(str_get_cols(snd_list(isonde),17,18))
dd = toint(str_get_cols(snd_list(isonde),19,20))
hh = toint(str_get_cols(snd_list(isonde),22,23))
mn = toint(str_get_cols(snd_list(isonde),24,25))

;print("sonde time is "+yy+" years, "+mm+" months, "+dd+" days, "+hh+" hours, and "+mn+" minutes.")

 sonde_times(isonde) = (/dble2flt(cd_inv_calendar(yy,mm,dd,hh,mn,00,units,0))/)
 
 ;; Read Sonde
 u_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_uv,nzp,"u")
 v_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_uv,nzp,"v")
 q_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_uv,nzp,"qv")
 t_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_uv,nzp,"t")
 p_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_uv,nzp,"p")
 
 
 tv_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_tv,nzr,"tv")
 thv_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_tv,nzr,"thv")
 prass_sonde(isonde,:) = CW3Esonde_2_AROvars(snd_list(isonde),z_sonde_tv,nzr,"p")
 
 
end do

;printMinMax(prass_sonde,0)

;----------------------;;--------------------------------->
; Declare Metadata for sonde-interpolated arrays.

u_sonde@_FillValue = -9999.
u_sonde@units = "m s^-1"
u_sonde@description = "Zonal wind from sonde interpolated to 449 retrieval one levels"

v_sonde@_FillValue = -9999.
v_sonde@units = "m s^-1"
v_sonde@description = "Meridional wind from sonde interpolated to 449 retrieval one levels"

q_sonde@_FillValue = -9999.
q_sonde@units = "kg kg^-1"
q_sonde@description = "wv mix. ratio from sonde interpolated to 449 retrieval one levels"

t_sonde@_FillValue = -9999.
t_sonde@units = "degrees C"
t_sonde@description = "temperature from sonde interpolated to 449 retrieval one levels"

p_sonde@_FillValue = -9999.
p_sonde@units = "hPa"
p_sonde@description = "pressure from sonde interpolated to 449 retrieval one levels"

prass_sonde@_FillValue = -9999.
prass_sonde@units = "hPa"
prass_sonde@description = "pressure from sonde interpolated to RASS levels"

tv_sonde@_FillValue = -9999.
tv_sonde@units = "K"
tv_sonde@description = "virtual temperature derived from sonde and interpolated to RASS levels"

thv_sonde@_FillValue = -9999.
thv_sonde@units = "K"
thv_sonde@description = "virtual pot. temp. derived from sonde and interpolated to RASS levels"





;-------------------------------;;---------------------------------->
; Write data to NetCDF file.
;-------------------------------;;---------------------------------->

NCFILE = "PSD-Met-SSS-YRRMNNDDY.nc"


   globeAtt                = 1
   globeAtt@title          = NCFILE
   globeAtt@creation_date  = systemfunc("date")

   print(""+NCFILE)

   system ("/bin/rm -f " + NCFILE)


   ncdf = addfile(NCFILE, "c")

   fileattdef( ncdf, globeAtt )

   dimNames = (/"Time1", "Time2", "Height", "RassHeight"/)
   dimSizes = (/ nhrs, nsondes, nzp, nzr /)
   dimUnlim = (/ False, False, False, False /)
   filedimdef(ncdf, dimNames, dimSizes, dimUnlim)

; Give variables names, pseudonames, and dimensions

   filevardef(ncdf, "Latitude", typeof(latitude), "ncl_scalar")
   filevarattdef(ncdf, "Latitude", latitude)
   
   filevardef(ncdf, "Longitude", typeof(longitude), "ncl_scalar")
   filevarattdef(ncdf, "Longitude", longitude)
   
   filevardef(ncdf, "Elevation", typeof(elevation), "ncl_scalar")
   filevarattdef(ncdf, "Elevation", elevation)
   
   filevardef(ncdf, "Upslope", typeof(upslope), "ncl_scalar")
   filevarattdef(ncdf, "Upslope", upslope)

   filevardef (ncdf, "Times" , typeof(fulltimes), (/ "Time1" /) )
   filevarattdef (ncdf, "Times", fulltimes(:) )
   
   filevardef (ncdf, "SondeTimes", typeof(sonde_times), (/ "Time2" /) )
   filevarattdef (ncdf, "SondeTimes", sonde_times(:) ) 
   
   filevardef (ncdf, "Height_retr1", typeof(z449_retr1), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "Height_retr1", z449_retr1(:,:) )

   filevardef (ncdf, "Height_retr2", typeof(z449_retr2), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "Height_retr2", z449_retr2(:,:) )
   
   filevardef (ncdf, "UWind_retr1", typeof(u449_retr1), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "UWind_retr1", u449_retr1(:,:) )

   filevardef (ncdf, "UWind_retr2", typeof(u449_retr2), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "UWind_retr2", u449_retr2(:,:) )
   
   filevardef (ncdf, "VWind_retr1", typeof(v449_retr1), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "VWind_retr1", v449_retr1(:,:) )

   filevardef (ncdf, "VWind_retr2", typeof(v449_retr2), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "VWind_retr2", v449_retr2(:,:) )
   
   filevardef (ncdf, "QC_retr1", typeof(qc449_retr1), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "QC_retr1", qc449_retr1(:,:) )

   filevardef (ncdf, "QC_retr2", typeof(qc449_retr2), (/ "Time1", "Height" /) )
   filevarattdef (ncdf, "QC_retr2", qc449_retr2(:,:) )
   
   filevardef (ncdf, "Height_RASS", typeof(zrass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "Height_RASS", zrass(:,:) )
   
   filevardef (ncdf, "Pressure_RASS", typeof(prass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "Pressure_RASS", prass(:,:) )
   
   filevardef (ncdf, "Virtual_Temp", typeof(trass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "Virtual_Temp", trass(:,:) )
   
   filevardef (ncdf, "Virtual_Theta", typeof(thrass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "Virtual_Theta", thrass(:,:) )
   
   filevardef (ncdf, "Vertical_Velocity", typeof(wrass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "Vertical_Velocity", wrass(:,:) )
   
   filevardef (ncdf, "QC_RASS_Tv", typeof(qc_trass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "QC_RASS_Tv", qc_trass(:,:) )
   
   filevardef (ncdf, "QC_RASS_w", typeof(qc_wrass), (/ "Time1", "RassHeight" /) )
   filevarattdef (ncdf, "QC_RASS_w", qc_wrass(:,:) )
   
   filevardef (ncdf, "Sonde_UWind", typeof(u_sonde), (/ "Time2", "Height" /) )
   filevarattdef (ncdf, "Sonde_UWind", u_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_VWind", typeof(v_sonde), (/ "Time2", "Height" /) )
   filevarattdef (ncdf, "Sonde_VWind", v_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_Qv", typeof(q_sonde), (/ "Time2", "Height" /) )
   filevarattdef (ncdf, "Sonde_Qv", q_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_Tc", typeof(t_sonde), (/ "Time2", "Height" /) )
   filevarattdef (ncdf, "Sonde_Tc", t_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_Press", typeof(p_sonde), (/ "Time2", "Height" /) )
   filevarattdef (ncdf, "Sonde_Press", p_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_PRass", typeof(prass_sonde), (/ "Time2", "RassHeight" /) )
   filevarattdef (ncdf, "Sonde_PRass", prass_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_Tv", typeof(tv_sonde), (/ "Time2", "RassHeight" /) )
   filevarattdef (ncdf, "Sonde_Tv", tv_sonde(:,:) )
   
   filevardef (ncdf, "Sonde_Thv", typeof(thv_sonde), (/ "Time2", "RassHeight" /) )
   filevarattdef (ncdf, "Sonde_Thv", thv_sonde(:,:) )
   
   filevardef (ncdf, "Pres_Sfc", typeof(pres_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "Pres_Sfc", pres_sfc(:) )
   
   filevardef (ncdf, "Temp_Sfc", typeof(temp_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "Temp_Sfc", temp_sfc(:) )
   
   filevardef (ncdf, "RH_Sfc", typeof(rh_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "RH_Sfc", rh_sfc(:) )
   
   filevardef (ncdf, "Wspd_Sfc", typeof(wspd_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "Wspd_Sfc", wspd_sfc(:) )
   
   filevardef (ncdf, "Wdir_Sfc", typeof(wdir_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "Wdir_Sfc", wdir_sfc(:) )
   
   filevardef (ncdf, "Solrad_Sfc", typeof(solrad_sfc), (/ "Time1" /) )
   filevarattdef (ncdf, "Solrad_Sfc", solrad_sfc(:) )
   
   filevardef (ncdf, "Precip", typeof(precip), (/ "Time1" /) )
   filevarattdef (ncdf, "Precip", precip(:) )
   
   filevardef (ncdf, "Gps_Iwv", typeof(gpsiwv), (/ "Time1" /) )
   filevarattdef (ncdf, "Gps_Iwv", gpsiwv(:) )
   
   filevardef (ncdf, "Bulk_IWV_Flux", typeof(bulkiwvflux), (/ "Time1" /) )
   filevarattdef (ncdf, "Bulk_IWV_Flux", bulkiwvflux(:) )  

   print(""+u449_retr1(:,2))

   print("    ")
   print("writing " +NCFILE)

   ncdf->Latitude               = (/ latitude /)
   ncdf->Longitude              = (/ longitude /)
   ncdf->Elevation              = (/ elevation /)
   ncdf->Upslope                = (/ upslope /)
   ncdf->Times                  = (/ fulltimes(:) /)
   ncdf->SondeTimes             = (/ sonde_times(:) /)
   ncdf->Height_retr1           = (/ z449_retr1(:,:) /)
   ncdf->Height_retr2           = (/ z449_retr2(:,:) /)
   ncdf->UWind_retr1            = (/ u449_retr1(:,:) /)
   ncdf->UWind_retr2            = (/ u449_retr2(:,:) /)
   ncdf->VWind_retr1            = (/ v449_retr1(:,:) /)
   ncdf->VWind_retr2            = (/ v449_retr2(:,:) /)
   ncdf->QC_retr1               = (/ qc449_retr1(:,:) /)
   ncdf->QC_retr2               = (/ qc449_retr2(:,:) /)
   ncdf->Height_RASS            = (/ zrass(:,:)  /)
   ncdf->Pressure_RASS          = (/ prass(:,:)  /)
   ncdf->Virtual_Temp           = (/ trass(:,:)  /)
   ncdf->Virtual_Theta          = (/ thrass(:,:) /)
   ncdf->Vertical_Velocity      = (/ wrass(:,:)  /)
   ncdf->QC_RASS_Tv             = (/ qc_trass(:,:) /)
   ncdf->QC_RASS_w              = (/ qc_wrass(:,:) /)
   ncdf->Sonde_UWind            = (/ u_sonde(:,:) /)
   ncdf->Sonde_VWind            = (/ v_sonde(:,:) /)
   ncdf->Sonde_Qv               = (/ q_sonde(:,:) /)
   ncdf->Sonde_Tc               = (/ t_sonde(:,:) /)
   ncdf->Sonde_Press            = (/ p_sonde(:,:) /)
   ncdf->Sonde_PRass            = (/ prass_sonde(:,:) /)
   ncdf->Sonde_Tv               = (/ tv_sonde(:,:) /)
   ncdf->Sonde_Thv              = (/ thv_sonde(:,:) /)
   ncdf->Pres_Sfc               = (/ pres_sfc(:) /)
   ncdf->Temp_Sfc               = (/ temp_sfc(:) /)
   ncdf->RH_Sfc                 = (/ rh_sfc(:) /)
   ncdf->Wspd_Sfc               = (/ wspd_sfc(:) /)
   ncdf->Wdir_Sfc               = (/ wdir_sfc(:) /)
   ncdf->Solrad_Sfc             = (/ solrad_sfc(:) /)
   ncdf->Precip                 = (/ precip(:) /)
   ncdf->Gps_Iwv                = (/ gpsiwv(:) /)
   ncdf->Bulk_IWV_Flux          = (/ bulkiwvflux(:) /)
   

   print(" writing complete ")





end 


